=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2007-04-11 09:10:45 +0000
+++ bzrlib/remote.py	2007-04-11 10:07:02 +0000
@@ -576,15 +576,25 @@
         return self._real_repository.check(revision_ids)
 
     def copy_content_into(self, destination, revision_id=None, basis=None):
+        # get a tarball of the remote repository, and copy from that into the
+        # destination
+        from bzrlib import osutils
         import tarfile
+        import tempfile
         from StringIO import StringIO
-        tempfile = StringIO(self._get_tarball('bz2'))
-        tar = tarfile.open('repository', fileobj=tempfile,
+        tar_file = StringIO(self._get_tarball('bz2'))
+        tar = tarfile.open('repository', fileobj=tar_file,
             mode='r:bz2')
-        import pdb;pdb.set_trace()
-        self._ensure_real()
-        return self._real_repository.copy_content_into(
-            destination, revision_id=revision_id, basis=basis)
+        tmpdir = tempfile.mkdtemp()
+        try:
+            tar.extractall(tmpdir)
+            tmp_repo = repository.Repository.open(tmpdir)
+            tmp_repo.copy_content_into(destination, revision_id, basis)
+        finally:
+            pass
+            ##            osutils.rmtree(tmpdir)
+        # TODO: if the server doesn't support this operation, maybe do it the
+        # slow way using the _real_repository?
 
     def set_make_working_trees(self, new_value):
         raise NotImplementedError(self.set_make_working_trees)

=== modified file 'bzrlib/smart/repository.py'
--- bzrlib/smart/repository.py	2007-04-11 09:10:45 +0000
+++ bzrlib/smart/repository.py	2007-04-11 10:07:02 +0000
@@ -198,8 +198,8 @@
         repo_transport = repository.control_files._transport
         tmp_dirname, tmp_repo = self._copy_to_tempdir(repository)
         try:
-            repo_dir = tmp_repo.control_files._transport.local_abspath('.')
-            return self._tarfile_response(repo_dir, compression)
+            controldir_name = tmp_dirname + '/.bzr'
+            return self._tarfile_response(controldir_name, compression)
         finally:
             osutils.rmtree(tmp_dirname)
 
@@ -210,10 +210,10 @@
         from_repo.copy_content_into(tmp_repo)
         return tmp_dirname, tmp_repo
 
-    def _tarfile_response(self, repo_dir, compression):
+    def _tarfile_response(self, tmp_dirname, compression):
         temp = tempfile.NamedTemporaryFile()
         try:
-            self._tarball_of_dir(repo_dir, compression, temp.name)
+            self._tarball_of_dir(tmp_dirname, compression, temp.name)
             # all finished; write the tempfile out to the network
             temp.seek(0)
             return SmartServerResponse(('ok',), temp.read())
@@ -222,18 +222,17 @@
         finally:
             temp.close()
 
-    def _tarball_of_dir(self, repo_dir, compression, tarfile_name):
+    def _tarball_of_dir(self, dirname, compression, tarfile_name):
         tarball = tarfile.open(tarfile_name, mode='w:' + compression)
         try:
             # The tarball module only accepts ascii names, and (i guess)
             # packs them with their 8bit names.  We know all the files
             # within the repository have ASCII names so the should be safe
             # to pack in.
-            repo_dir = repo_dir.encode(sys.getfilesystemencoding())
+            dirname = dirname.encode(sys.getfilesystemencoding())
             # python's tarball module includes the whole path by default so
             # override it
-            assert repo_dir.endswith('/repository'), \
-                "unexpected repository path %r" % repo_dir
-            tarball.add(repo_dir, 'repository') # recursive by default
+            assert dirname.endswith('.bzr')
+            tarball.add(dirname, '.bzr') # recursive by default
         finally:
             tarball.close()

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2007-04-11 09:10:45 +0000
+++ bzrlib/tests/test_remote.py	2007-04-11 10:07:02 +0000
@@ -654,20 +654,20 @@
     ##     dest_repo = self.make_repository('repo2')
     ##     src_repo.copy_content_into(dest_repo)
 
-    ## def test_detect_copy_using_tarball(self):
-    ##     self.transport_server = server.SmartTCPServer_for_testing
-    ##     src_repo = self.make_repository('repo1')
-    ##     src_repo = repository.Repository.open(self.get_url('repo1'))
-    ##     # At the moment the tarball-based copy_content_into can't write back
-    ##     # into a smart server.  It would be good if it could upload the
-    ##     # tarball; once that works we'd have to create repositories of
-    ##     # different formats. -- mbp 20070410
-    ##     dest_url = self.get_vfs_only_url('repo2')
-    ##     dest_bzrdir = BzrDir.create(dest_url)
-    ##     dest_repo = dest_bzrdir.create_repository()
-    ##     self.assertFalse(isinstance(dest_repo, RemoteRepository))
-    ##     self.assertTrue(isinstance(src_repo, RemoteRepository))
-    ##     src_repo.copy_content_into(dest_repo)
+    def test_copy_content_remote_to_local(self):
+        self.transport_server = server.SmartTCPServer_for_testing
+        src_repo = self.make_repository('repo1')
+        src_repo = repository.Repository.open(self.get_url('repo1'))
+        # At the moment the tarball-based copy_content_into can't write back
+        # into a smart server.  It would be good if it could upload the
+        # tarball; once that works we'd have to create repositories of
+        # different formats. -- mbp 20070410
+        dest_url = self.get_vfs_only_url('repo2')
+        dest_bzrdir = BzrDir.create(dest_url)
+        dest_repo = dest_bzrdir.create_repository()
+        self.assertFalse(isinstance(dest_repo, RemoteRepository))
+        self.assertTrue(isinstance(src_repo, RemoteRepository))
+        src_repo.copy_content_into(dest_repo)
 
     ## def test_copy_content_into_uses_tarball(self):
     ##     # copy_content_into on a RemoteRepository should try to use the

